require 'httpclient'
require 'cgi'

module Micropub
  class ValidationError < StandardError;end

  class Token
    def initialize(token)
      @token = token
    end

    def valid?
      begin
        valid_host? &&
        valid_scope?
      rescue ValidationError
        false
      end
    end

    def valid_scope?
      (config.allowed_scopes & scope).any?
    end

    def valid_host?
      me.host == config.me.host
    end

    def scope
      response["scope"].split(",").map(&:to_sym)
    end

    def me
      begin
        URI.parse(response["me"])
      rescue URI::InvalidURIError
        raise ValidationError, "The token endpoint did not contain a valid 'me' URI"
      end
    end

    def response
      @response ||= begin
        URI::decode_www_form(request.body).to_h
      end
    end

    def request
      HTTPClient.get(config.token_endpoint, {}, {'Authorization' => "Bearer #{@token}"})
    end

    def config
      Micropub.configuration
    end
  end
end
